@@ -1,5 +1,5 @@
 
-/*  Copyright (c) 1987-2006 Sun Microsystems, Inc. All Rights Reserved.
+/*  Copyright (c) 1987-2007 Sun Microsystems, Inc. All Rights Reserved.
  *           
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -177,7 +177,15 @@ calc_xpowy(int MPx[MP_SIZE], int MPy[MP_SIZE], int MPres[MP_SIZE]) /* Do x^y */
 
     val = 0;
     mpcim(&val, MP0);
-    if (mplt(MPx, MP0)) {          /* Is x < 0 ? */
+
+    /* Check if both x and y are zero. If yes, then just return 1.
+     * See gcalctool bug #451286.
+     */
+    if (mpeq(MPx, MP0) && mpeq(MPy, MP0)) {
+        val = 1;
+        mpcim(&val, MPres);
+
+    } else if (mplt(MPx, MP0)) {          /* Is x < 0 ? */
         int MPtmp[MP_SIZE];
 
         mpcmim(MPy, MPtmp);
@@ -225,17 +233,14 @@ calc_modulus(int op1[MP_SIZE],
 }
 
 void
-calc_percent(int s1[MP_SIZE], int s2[MP_SIZE], int t1[MP_SIZE])
+calc_percent(int s1[MP_SIZE], int t1[MP_SIZE])
 {
     int MP1[MP_SIZE];
-    int MP2[MP_SIZE];
 
-    mpmul(s1, s2, MP1);
-    MPstr_to_num("0.01", DEC, MP2);
-    mpmul(MP1, MP2, t1);
+    MPstr_to_num("0.01", DEC, MP1);
+    mpmul(s1, MP1, t1);
 }
 
-
 void
 do_zero(int t1[MP_SIZE])
 {
@@ -820,3 +825,32 @@ calc_rshift(int s[MP_SIZE], int t[MP_SIZE], int times, enum shiftd dir)
     dval = setbool(temp);
     mpcdm(&dval, t);
 }
+
+
+int
+is_integer(int MPnum[MP_SIZE])
+{    
+    int MP1[MP_SIZE];
+    mpcmim(MPnum, MP1);
+    return mpeq(MPnum, MP1);
+}
+
+int
+is_natural(int MPnum[MP_SIZE])
+{    
+    int MP1[MP_SIZE];
+    if (!is_integer(MPnum)) {
+	return 0;
+    }
+    mpabs(MPnum, MP1);
+    return mpeq(MPnum, MP1);
+}
+
+void
+calc_epowy(int s[MP_SIZE], int t[MP_SIZE])
+{
+    int MP1[MP_SIZE];
+    
+    mpstr(s, MP1);
+    mpexp(MP1, t);
+}
